// Rozsireni o naklonomer MPU-6050
//================================================


#ifdef naklonomer

int16_t AXp, AYp, AZp;                                                   // zprumerovane tihove zrychleni v kazde ose
int16_t offset_x, offset_y, offset_z;                                    // hodnoty offsetu pro kalibraci nulove polohy

int16_t OFFS_write;                                                      // hodnota pro zapis do jednotlivych offsetovych registru


int32_t X_bod_K_A, Y_bod_K_A, Z_bod_K_A;                                 // hodnota zrychleni pro body K ve vsech osach
int32_t X_bod_L_A, Y_bod_L_A, Z_bod_L_A;                                 // hodnota zrychleni pro body L ve vsech osach
int32_t X_bod_K_O, Y_bod_K_O, Z_bod_K_O;                                 // hodnota offsetu pro body K ve vsech osach
int32_t X_bod_L_O, Y_bod_L_O, Z_bod_L_O;                                 // hodnota offsetu pro body L ve vsech osach


int16_t nak_cyklus;                                                      // pomocna promenna pro ruzne smycky

float nak_kalibrace0;                                                    // slouzi k premapovani uhlu, ktery byl spocteny pri horizontalni pozici na 0 stupnu
float nak_kalibrace90;                                                   // slouzi k premapovani uhlu, ktery byl spocteny pri vertikallni pozici na 90 stupnu

int16_t zen_0_lp, zen_0_pz;                                              // pro spravnou funkci vertikalni vodovahy se musi zaznamenat kalibracni hodnoty dvou os pri vertikalni poloze krabicky


//----------------------------------------------
// pocatecni nastaveni naklonomeru vcetne dat z posledni kalibrace
void naklon_setup(void)
  {

    Wire.beginTransmission (I2C_ADDR_NAKLON);                            // pocatecni reset a zakazani teplomeru
    Wire.write (0x6B);
    Wire.write (0b00001000);
    Wire.endTransmission (true);

    delay(100);

    Wire.beginTransmission (I2C_ADDR_NAKLON);                            // nastaveni nejcitlivejsiho rozsahu zrychleni (+/- 2g)
    Wire.write (0x1C);
    Wire.write(0x00);
    Wire.endTransmission (true);

    delay(20);

    Wire.beginTransmission (I2C_ADDR_NAKLON);                            // filtrace signalu (snizeni sumu)
    Wire.write (0x1D);
    Wire.write(0b00000110);
    Wire.endTransmission (true);

    delay(20);


    int kalibracni_uhel = EEPROM_read_int(477);                          // uhel, ktery se ma premapovavat na horizont = 0 stupnu (adresa 477 a 478)
    nak_kalibrace0 = (kalibracni_uhel - 10000) / 100.0;                  // do EEPROM se tyto uhly ukladaji jako cele cislo v setinach stupne a jeste posunute o 100°, aby bylo vzdycky zajisteno ukladani kladneho cisla
    
    kalibracni_uhel = EEPROM_read_int(479);                              // uhel, ktery se ma premapovavat na zenit = 90 stupnu (adresa 479 az 480)   
    nak_kalibrace90 = (kalibracni_uhel - 10000) / 100.0;

    zen_0_lp = EEPROM_read_int(481);                                     // s temito dvema hodnotami se bude porovnavat aktualni hodnota zrychleni ve dvou osach pri funkci vertikalni vodovahy 
    zen_0_pz = EEPROM_read_int(483);

    if (nak_kalibrace0 == nak_kalibrace90)                               // kdyz jeste nebyla provedena kalibrace naklonomeru, tak se prednastavi alespon nejake nepresne hodnoty
      {
        default_naklonomer();
      }

    
    for (byte registr = 6; registr < 12 ; registr = registr + 2)         // precteni offsetovych hodnot z EEPROM a jejich zapis do MPU-6050
      {
        int16_t pomprom = EEPROM_read_int(registr + 465);                // offsety jsou ulozene v EEPROM na adresach 471 az 476
        Wire.beginTransmission (I2C_ADDR_NAKLON); 
        Wire.write (registr);
        Wire.write(pomprom / 256);
        Wire.endTransmission (true);
        delay(20);
        Wire.beginTransmission (I2C_ADDR_NAKLON); 
        Wire.write (registr+1);
        Wire.write(pomprom % 256);
        Wire.endTransmission (true);
        delay(20);                 
      }

    offset_x = EEPROM_read_int(471);
    offset_y = EEPROM_read_int(473);
    offset_z = EEPROM_read_int(475);
  
  
  }
//----------------------------------------------



//----------------------------------------------
uint16_t uhel(void)
  {
    prumer50();                                                          // do globalnich promennych AXp, AYp a AZp ulozi prumernou hodnotu zrychleni z 50 mereni
    float uhel_z = atan2(AZp, sqrt(square(AXp) + square(AYp)))/(PI/180);

    float vysledek = mapfloat(uhel_z, nak_kalibrace0 , nak_kalibrace90 , 0 , 90);    // premapovani: Zenit = 90, horizont = 0. Z pohledu cidla jsou tyto 2 hodnoty prohozene, proto se musi premapovat

#if ulozeni_naklonomeru == 3                                             // naklonomer s pohybem kolem osy Y (modul 3V3 s 10 vyvody)
    if (AXp <= 0) vysledek = - vysledek;                                 // osetreni znamenka u zapornych uhlu (namireni krabicky pod horizont)
#endif

#if ulozeni_naklonomeru == 4                                             // naklonomer s pohybem kolem osy Y (modul 3V3 s 10 vyvody)
    if (AXp > 0) vysledek = - vysledek;                                  // osetreni znamenka u zapornych uhlu (namireni krabicky pod horizont)
#endif

#if ulozeni_naklonomeru == 1                                             // naklonomer s pohybem kolem osy X (modul 5V s 8 vyvody)
    if (AYp > 0) vysledek = - vysledek;                                  // osetreni znamenka u zapornych uhlu (namireni krabicky pod horizont)
#endif


#if ulozeni_naklonomeru == 2                                             // naklonomer s pohybem kolem osy X (modul 5V s 8 vyvody)
    if (AYp > 0) vysledek = - vysledek;                                  // osetreni znamenka u zapornych uhlu (namireni krabicky pod horizont)
#endif

    vysledek =(vysledek + 0.05 + 1000.0) * 10.0;                         // prevod na kladne cislo (posun o 1000 stupnu), zaokrouhleni na 1 desetinu a prevod na cele cislo

    return vysledek;                    
    
  }
//----------------------------------------------



//----------------------------------------------
// prumerovani 50 vzorku zrychleni ve vsech osach
//  vysledek je ulozeny v globalnich promennych AXp, AYp a AZp
void prumer50(void)
  {
    int32_t sumaX = 0;
    int32_t sumaY = 0;
    int32_t sumaZ = 0; 
    byte smycka;
    
    for (smycka = 0; smycka < 50; smycka++)                              // prumerovani 50 hodnot
      {      
        Wire.beginTransmission (I2C_ADDR_NAKLON);
        Wire.write (0x3B);                                               // uvodni registr pro zrychleni v ose X (2 bajty)
        Wire.endTransmission (false);
        delayMicroseconds(10);
        Wire.requestFrom (I2C_ADDR_NAKLON, 6, true);                     // celkem 6 registru (2 pro kazdou osu)
        delayMicroseconds(10);

        int16_t pomprom_xyz = (Wire.read() << 8 | Wire.read());          // aktualni hodnoty se nascitavaji do sumarnich promennych
        sumaX = sumaX + pomprom_xyz;                             

        pomprom_xyz = (Wire.read() << 8 | Wire.read());
        sumaY = sumaY + pomprom_xyz;

        pomprom_xyz = (Wire.read() << 8 | Wire.read());
        sumaZ = sumaZ + pomprom_xyz;
      }
    
    AXp = sumaX / 50;                                                    // zprumerovane hodnoty
    AYp = sumaY / 50;
    AZp = sumaZ / 50;
    delay(80);
  }
//----------------------------------------------



//----------------------------------------------
void default_naklonomer(void)
  {

#if ulozeni_naklonomeru == 1                                             // modul s 8 vyvody cipem nahoru
    EEPROM_write_int(471 , 65266);                                       // offset X (-270)
    EEPROM_write_int(473 ,   370);                                       // offset Y
    EEPROM_write_int(475 ,  1180);                                       // offset Z

    EEPROM_write_int(481 , 65279);                                       // nulova hodnota pri naklaneni leva/prava je ve vertikalni pozici asi -257
    EEPROM_write_int(483 , 65050);                                       // nulova hodnota pri naklaneni predni/zadni je ve vertikalni pozici asi -486
    zen_0_lp = -257;
    zen_0_pz = -486;
#endif
    



#if ulozeni_naklonomeru == 2                                             // modul s 8 vyvody cipem dolu
    EEPROM_write_int(471 , 65266);                                       // offset X (-270)
    EEPROM_write_int(473 ,   370);                                       // offset Y
    EEPROM_write_int(475 ,  1180);                                       // offset Z

    EEPROM_write_int(481 ,   882);                                       // nulova hodnota pri naklaneni leva/prava je ve vertikalni pozici asi 882
    EEPROM_write_int(483 ,   426);                                       // nulova hodnota pri naklaneni predni/zadni je ve vertikalni pozici asi 426
    zen_0_lp = 882;
    zen_0_pz = 426;
#endif


#if ulozeni_naklonomeru == 3                                             // modul s 10 vyvody nalepeny na piskaku - otaceni kolem osy y
    EEPROM_write_int(471 , 65236);                                       // offset X (-300)
    EEPROM_write_int(473 ,   320);                                       // offset Y
    EEPROM_write_int(475 ,  1148);                                       // offset Z

    EEPROM_write_int(481 , 65159);                                       // nulova hodnota pri naklaneni leva/prava je ve vertikalni pozici asi -377
    EEPROM_write_int(483 , 65149);                                       // nulova hodnota pri naklaneni predni/zadni je ve vertikalni pozici asi -387
    zen_0_lp = -377;
    zen_0_pz = -387;
#endif



#if ulozeni_naklonomeru == 4                                             // modul s 10 vyvody pripajeny primo na desku - otaceni kolem osy y
    EEPROM_write_int(471 , 65236);                                       // offset X (-300)
    EEPROM_write_int(473 ,   320);                                       // offset Y
    EEPROM_write_int(475 ,  1148);                                       // offset Z

    EEPROM_write_int(481 , 65395);                                       // nulova hodnota pri naklaneni leva/prava je ve vertikalni pozici asi -141
    EEPROM_write_int(483 , 65423);                                       // nulova hodnota pri naklaneni predni/zadni je ve vertikalni pozici asi -113
    zen_0_lp = -141;
    zen_0_pz = -113;
#endif






#if ulozeni_naklonomeru == 2
    EEPROM_write_int(477 , 1000);                                        // horizont je z pohledu cidla na -90 stupnich. Premapovani na 0 se provadi az pri vypoctu aktualniho uhlu
    nak_kalibrace0 = -90;                                                //  (pri ulozeni cipem dolu je horizont -90 stupnu)
#endif

#if ulozeni_naklonomeru == 1 || ulozeni_naklonomeru == 3 || ulozeni_naklonomeru == 4 
    EEPROM_write_int(477 , 19000);                                       // horizont je z pohledu cidla na 90 stupnich. Premapovani na 0 se provadi az pri vypoctu aktualniho uhlu
    nak_kalibrace0 = 90;                                                 //   (pri ostatnich ulozenich je horizont +90 stupnu)
#endif

    EEPROM_write_int(479 , 10000);                                       // zenit je z pohledu cidla vzdycky na 0 stupnich. Premapovani na 90 se provadi az pri vypoctu aktualniho uhlu
    nak_kalibrace90 = 0;
    

    for (byte registr = 6; registr < 12 ; registr = registr + 2)         // precteni offsetovych hodnot z EEPROM a jejich zapis do MPU-6050
      {
        int16_t pomprom = EEPROM_read_int(registr + 465);                // offsety jsou ulozene v EEPROM na adresach 471 az 476
        Wire.beginTransmission (I2C_ADDR_NAKLON); 
        Wire.write (registr);
        Wire.write(pomprom / 256);
        Wire.endTransmission (true);
        delay(20);
        Wire.beginTransmission (I2C_ADDR_NAKLON); 
        Wire.write (registr+1);
        Wire.write(pomprom % 256);
        Wire.endTransmission (true);
        delay(20);                 
      }

                

    
  }


//----------------------------------------------
// Automaticka kalibrace naklonomeru
//  Probiha nastaveni akcelerometrovych offsetu ve vsech trech osach pro horizontalni polohu
//  Na zaver se z prectenych hodnot zrychleni vypocte uhel, na který se bude premapovavat na 0 stupnu.
//  Pak se SQM natoci do zenitu a změri se druhy uhel, ktery se bude premapovavat na 90 stupnu.
//  Vsechny offsety a oba koncove body se zapisuji do EEPROM (adresy 471 az 480).
//  Pri zapnuti napajeni jsou hodnoty z teto pameti vybrany a pouzity pro vypocty.
void kalibrace_naklon(boolean dbg)
  {

    byte X_konec=0, Y_konec=0, Z_konec=0;                                // ktery konec kalibracni usecky se hleda (0 = prvni konec, 1 = druhy konec, 2 = hotovo, dal se uz nehleda)

    zobraz_text(47);                                                     // "n-CAL" (kalibrace naklonomeru)
    
    Serial.println(lng232);                                              // "Poloz SQM horizontalne"

    Serial.println(">>");
    while (Serial.available() == 0)  delay(100);                         // ceka na odklepnuti horizontove polohy
    while(Serial.available()) Serial.read();



//--------------------  FAZE 1 -----------------------
// do vsech registru se budou zapisovat vzrustajici offsety a bude se hledat takova situace, kdy zrychleni v osach x a y prekroci
//     hodnoty 4000 a 14000  (osa z je trochu jina, tam se hleda, kdy zrychleni prekroci hodnoty 12000 a 20000)
//     Vysledkem teto faze jsou pro kazdy smer 2 body (K,L) usecky.


    if (dbg == false)
      {
        Serial.println("[                    |   |   |   ]");            // jenom bargraf pod kterym se kazdych 400 cisel zobrazi hvezdicka
        Serial.print('[');    
      }

    for (nak_cyklus = -4000; nak_cyklus < 4000; nak_cyklus = nak_cyklus + 50)        // nejdriv se na hrubo najde takovy offset, pri kterem prekroci vystupy AXp a AYp hodnoty 4000 a 14000 
      {                                                                              // (pro osu z se testuje vystup AZp na hodnoty 12000 a 20000)


        if (nak_cyklus < 0 )  OFFS_write = nak_cyklus - 256;             // pro zaporne hodnoty zapisovanych offsetu se musi odecist cislo 256
        else                  OFFS_write = nak_cyklus;

        // Zapis hodnoty offsetu do vnitrnich registru cipu MPU6050        
        if (X_konec < 2)   zapis_offset(OFFS_write , 6);                 // offset X se zapisuje jen tak dlouho, nez jsou nalezene oba body
        if (Y_konec < 2)   zapis_offset(OFFS_write , 8);                 // offset Y se zapisuje jen tak dlouho, nez jsou nalezene oba body
        if (Z_konec < 2)   zapis_offset(OFFS_write , 10);                // offset Z se zapisuje jen tak dlouho, nez jsou nalezene oba body

        if (X_konec + Y_konec + Z_konec != 6)                            // mereni a vypocty se provadi jed do te doby, dokud nejsou nalezeny oba body ve vsech osach
          {
            prumer50();
          }

        if (nak_cyklus % 400 == 0) 
          {
            if (dbg == false)
              {
                Serial.print('*');
              }
            else
              {
                Serial.print("Offset= ");
                Serial.print(OFFS_write);
                Serial.print("      AXp= ");
                Serial.print(AXp);
                Serial.print("      AYp= ");
                Serial.print(AYp);
                Serial.print("      AZp= ");
                Serial.println(AZp);
              }            
          }


        if (AXp > 4000 and X_konec == 0)                                 // hledani prvniho konce usecky (bod 'K') pro osu X
          {
            X_bod_K_A = AXp;
            X_bod_K_O = OFFS_write;
            X_konec = 1;
          }

        if (AXp > 14000 and X_konec == 1)                                // hledani druheho konce usecky (bod 'L') pro osu X
          {
            X_bod_L_A = AXp;
            X_bod_L_O = OFFS_write;
            X_konec = 2;
          }


        if (AYp > 4000 and Y_konec == 0)                                 // hledani prvniho konce usecky (bod 'K') pro osu Y
          {
            Y_bod_K_A = AYp;
            Y_bod_K_O = OFFS_write;
            Y_konec = 1;
          }

        if (AYp > 14000 and Y_konec == 1)                                // hledani druheho konce usecky (bod 'L') pro osu Y
          {
            Y_bod_L_A = AYp;
            Y_bod_L_O = OFFS_write;
            Y_konec = 2;
          }


#if ulozeni_naklonomeru == 1 || ulozeni_naklonomeru == 3 || ulozeni_naklonomeru == 4                 // kalibrace horizontalni polohy pro cip umisteny smerem vzhuru
        if (AZp > 12000 and Z_konec == 0)                                // hledani prvniho konce usecky (bod 'K') pro osu Z (proti osam X a Y jsou tady body trochu posunute)
          {
            Z_bod_K_A = AZp;
            Z_bod_K_O = OFFS_write;
            Z_konec = 1;
          }

        if (AZp > 20000 and Z_konec == 1)                                // hledani druheho konce usecky (bod 'L') pro osu Z
          {
            Z_bod_L_A = AZp;
            Z_bod_L_O = OFFS_write;
            Z_konec = 2;
          }
#endif

#if ulozeni_naklonomeru == 2                                             // kalibrace horizontalni polohy pro cip umisteny smerem dolu
        if (AZp > -20000 and Z_konec == 0)                               // hledani prvniho konce usecky (bod 'K') pro osu Z (proti osam X a Y jsou tady body trochu posunute)
          {
            Z_bod_K_A = AZp;
            Z_bod_K_O = OFFS_write;
            Z_konec = 1;
          }

        if (AZp > -12000 and Z_konec == 1)                               // hledani druheho konce usecky (bod 'L') pro osu Z
          {
            Z_bod_L_A = AZp;
            Z_bod_L_O = OFFS_write;
            Z_konec = 2;
          }
#endif




      }

    if (dbg == false)
      {
        Serial.print('|');
      }
      

    if (X_konec + Y_konec + Z_konec != 6)
      {
        Serial.println(lng234);                                          // "Chyba kalibrace naklonu - nastavuji defaultni hodnoty"

        Serial.println(">>");
        while (Serial.available() == 0)  delay(100);                     // ceka na odklepnuti
        while(Serial.available()) Serial.read();

        default_naklonomer();                                            // nastaveni defaultnich offsetu a krajnich bodu

        return;                                                          // konec podprogramu - chyba autokalibrace
      }

//--------------------  FAZE 2 -----------------------
// ze dvou znamych bodu kazde usecky se nahrubo vypocte takovy offset, pri kterem usecka protina osu x  (vystup zrychleni je 0)
//  vychazi se z obecne rovnice primky y = ax + b
      
    float a = (float)(X_bod_L_A - X_bod_K_A) / (X_bod_L_O - X_bod_K_O);
    float b = X_bod_K_A - (X_bod_K_O * a);
    float TeorOffX = -b / a;                                             // 'TeorOffX' je teoreticka hodnota ve ktere by mela primka mezi body K a L protinat osou x (hodnota na ose y = 0) 
    if (dbg == false)
      {
        Serial.print('*');
      }
    else
      {
        Serial.print("xK = [");
        Serial.print(X_bod_K_O);
        Serial.print(',');
        Serial.print(X_bod_K_A);
        Serial.println(']');

        Serial.print("xL = [");
        Serial.print(X_bod_L_O);
        Serial.print(',');
        Serial.print(X_bod_L_A);
        Serial.println(']');

        Serial.print("yK = [");
        Serial.print(Y_bod_K_O);
        Serial.print(',');
        Serial.print(Y_bod_K_A);
        Serial.println(']');

        Serial.print("yL = [");
        Serial.print(Y_bod_L_O);
        Serial.print(',');
        Serial.print(Y_bod_L_A);
        Serial.println(']');

        Serial.print("zK = [");
        Serial.print(Z_bod_K_O);
        Serial.print(',');
        Serial.print(Z_bod_K_A);
        Serial.println(']');

        Serial.print("zL = [");
        Serial.print(Z_bod_L_O);
        Serial.print(',');
        Serial.print(Z_bod_L_A);
        Serial.println(']');
        
        Serial.print("TeorOffX:");
        Serial.println(TeorOffX);
      }

    a = (float)(Y_bod_L_A - Y_bod_K_A) / (Y_bod_L_O - Y_bod_K_O);
    b = Y_bod_K_A - (Y_bod_K_O * a);
    float TeorOffY = -b / a;                                             // 'TeorOffY' je teoreticka hodnota ve ktere by mela primka mezi body K a L protinat osou x (hodnota na ose y = 0) 
    if (dbg == false)
      {
        Serial.print('*');
      }
    else
      {
        Serial.print("TeorOffY:");
        Serial.println(TeorOffY);
      }

    a = (float)(Z_bod_L_A - Z_bod_K_A) / (Z_bod_L_O - Z_bod_K_O);        // osa Z se nastavuje tak, aby mela v horizontalni poloze hodnotu 16384, proto je vypocet trochu jiny
    b = Z_bod_K_A - (Z_bod_K_O * a);

#if ulozeni_naklonomeru == 1 || ulozeni_naklonomeru == 3 || ulozeni_naklonomeru == 4                 // kalibrace horizontalni polohy pro cip umisteny smerem vzhuru
    float TeorOffZ = (16384-b) / a;                                      // 'TeorOffZ' je teoreticka hodnota ve ktere by mela mit primka mezi body K a L hodnotu -16384 na ose y
#endif    

#if ulozeni_naklonomeru == 2                                             // kalibrace horizontalni polohy pro cip umisteny smerem dolu
    float TeorOffZ = (-16384-b) / a;                                     // 'TeorOffZ' je teoreticka hodnota ve ktere by mela mit primka mezi body K a L hodnotu 16384 na ose y
#endif    



    if (dbg == false)
      {
        Serial.print("*|");
      }
    else
      {
        Serial.print("TeorOffZ:");
        Serial.println(TeorOffZ);
      }


  
//--------------------  FAZE 3 -----------------------
//   Tady uz jde kazda osa ve sve vlastni smycce.
//   ke kazdemu offsetu se prida na hrubo nekolik desitek cisel, aby bylo zajisteno, ze vysledne zrychleni bude bezpecne kladne (nebo nad hodnotou 16384 pro osu Z)

    nak_cyklus = TeorOffX;                                               // zacina se teoreticky vypoctenou hodnotou offsetu pro osu X
    if (TeorOffX < 0) nak_cyklus = nak_cyklus - 256;                     // pro zaporny teoreticky offset se musi odecist jeste cislo 256
    do                                                                   // smycka musi projit alespon 1x
      {
        nak_cyklus = nak_cyklus + 20;                                    // k offsetu se pricita nekolik desitek cisel, dokud neni vystup zarucene kladny
        zapis_offset(nak_cyklus , 6);
         
        prumer50();
        if (dbg == true)
          {
            Serial.print("X: offset: ");
            Serial.print(nak_cyklus);
            Serial.print("     AXp: ");
            Serial.println(AXp);
          }
      }      
    while (AXp < 100);          
    offset_x = nak_cyklus;                                               // pomocna promenna pro osu X urcuje, od ktereho cisla se bude v nasledujici fazi pokracovat s jemnym doladenim
    if (dbg == false)
      {
        Serial.print('*');
      }


    nak_cyklus = TeorOffY;                                               // zacina se teoreticky vypoctenou hodnotou offsetu pro osu Y
    if (TeorOffY < 0) nak_cyklus = nak_cyklus - 256;                     // pro zaporny teoreticky offset se musi odecist jeste cislo 256

    do                                                                   // smycka musi projit alespon 1x
      {
        nak_cyklus = nak_cyklus + 20;                                    // k offsetu se pricita nekolik desitek cisel, dokud neni vystup zarucene kladny
        zapis_offset(nak_cyklus , 8);
        prumer50();
        if (dbg == true)
          {
            Serial.print("Y: offset: ");
            Serial.print(nak_cyklus);
            Serial.print("     AYp: ");
            Serial.println(AYp);
          }
      }      
    while (AYp < 100);          
    offset_y = nak_cyklus;                                               // pomocna promenna pro osu X urcuje, od ktereho cisla se bude v nasledujici fazi pokracovat s jemnym doladenim
    if (dbg == false)
      {
        Serial.print('*');
      }


    nak_cyklus = TeorOffZ;                                               // zacina se teoreticky vypoctenou hodnotou offsetu pro osu Z
    if (TeorOffZ < 0) nak_cyklus = nak_cyklus - 256;                     // pro zaporny teoreticky offset se musi odecist jeste cislo 256

#if ulozeni_naklonomeru == 1 || ulozeni_naklonomeru == 3 || ulozeni_naklonomeru == 4                 // kalibrace horizontalni polohy pro cip umisteny smerem vzhuru  
    do                                                                   // smycka musi projit alespon 1x
      {
        nak_cyklus = nak_cyklus + 20;                                    // k offsetu se pricita nekolik desitek cisel, dokud neni vystup zarucene nad pozadovanou hodnotou (pozaduje se 16384, hleda se 16500)
        zapis_offset(nak_cyklus , 10);
        prumer50();
        if (dbg == true)
          {
            Serial.print("Z: offset: ");
            Serial.print(nak_cyklus);
            Serial.print("     AZp: ");
            Serial.println(AZp);
          }
      }      
    while (AZp < 16500);          
#endif
    
#if ulozeni_naklonomeru == 2                                             // kalibrace horizontalni polohy pro cip umisteny smerem dolu  
    do                                                                   // smycka musi projit alespon 1x
      {
        nak_cyklus = nak_cyklus + 20;                                    // k offsetu se pricita nekolik desitek cisel, dokud neni vystup zarucene nad pozadovanou hodnotou (pozaduje se -16384, hleda se 16200)
        zapis_offset(nak_cyklus , 10);
        prumer50();
        if (dbg == true)
          {
            Serial.print("Z: offset: ");
            Serial.print(nak_cyklus);
            Serial.print("     AZp: ");
            Serial.println(AZp);
          }
      }      
    while (AZp < -16500);          
#endif


    
    offset_z = nak_cyklus;                                               // pomocna promenna pro osu Z urcuje, od ktereho cisla se bude v nasledujici fazi pokracovat s jemnym doladenim
    if (dbg == false)
      {
        Serial.print("*|");
      }

  

//--------------------  FAZE 4 -----------------------
//  Az sem se hodnota offsetu urcovala s velkou rychlosti, ale na hrubo.
//  Od ted uz se POMALU odecita od offsetu jednicka tak dlouho, nez se merene zrychleni dostane pod 0


    if (offset_x < 0) offset_x = offset_x + 256;
    nak_cyklus = offset_x;
    while (AXp >= 0)                                                     // pomale priblizovani k 0 na ose X zhora
      {
        nak_cyklus --;
        offset_x = nak_cyklus;
        if (offset_x < 0) offset_x = offset_x - 256;
        if (offset_x % 256 == 0) offset_x --;
        zapis_offset(offset_x , 6);
        prumer50();
        if (dbg == true)
          {
            Serial.print("X: offset: ");
            Serial.print(offset_x);
            Serial.print("     AXp: ");
            Serial.println(AXp);
          }

      }
    if (dbg == false)
      {
        Serial.print('*');
      }

    if (offset_y < 0) offset_y = offset_y + 256;
    nak_cyklus = offset_y;
    while (AYp >= 0)                                                     // pomale priblizovani k 0 na ose Y zhora
      {
        nak_cyklus --;
        offset_y = nak_cyklus;
        if (offset_y < 0) offset_y = offset_y - 256;
        if (offset_y % 256 == 0) offset_y --;

        zapis_offset(offset_y , 8);
        prumer50();
        if (dbg == true)
          {
            Serial.print("Y: offset: ");
            Serial.print(offset_y);
            Serial.print("     AYp: ");
            Serial.println(AYp);
          }
      }
    if (dbg == false)
      {
        Serial.print('*');
      }
      
    if (offset_z < 0) offset_z = offset_z + 256;
    nak_cyklus = offset_z;

#if ulozeni_naklonomeru == 1 || ulozeni_naklonomeru == 3 || ulozeni_naklonomeru == 4                 // kalibrace horizontalni polohy pro cip umisteny smerem vzhuru  
    while (AZp >= 16384)                                                  // pomale priblizovani k 16384 na ose Z zhora
      {
        nak_cyklus --;
        offset_z = nak_cyklus;
        if (offset_z < 0) offset_z = offset_z - 256;
        if (offset_z % 256 == 0) offset_z --;

        zapis_offset(offset_z , 10);
        prumer50();
        if (dbg == true)
          {
            Serial.print("Z: offset: ");
            Serial.print(offset_z);
            Serial.print("     AZp: ");
            Serial.println(AZp);
          }

      }
    if (dbg == false)
      {
        Serial.println("*]");
      }      
#endif

#if ulozeni_naklonomeru == 2                                             // kalibrace horizontalni polohy pro cip umisteny smerem dolu  
    while (AZp > -16384)                                                 // pomale priblizovani k 16384 na ose Z zhora
      {
        nak_cyklus --;
        offset_z = nak_cyklus;
        if (offset_z < 0) offset_z = offset_z - 256;
        if (offset_z % 256 == 0) offset_z --;

        zapis_offset(offset_z , 10);
        prumer50();
        if (dbg == true)
          {
            Serial.print("Z: offset: ");
            Serial.print(offset_z);
            Serial.print("     AZp: ");
            Serial.println(AZp);
          }

      }
    if (dbg == false)
      {
        Serial.println("*]");
      }      

#endif




  
    nak_kalibrace0 = atan2(AZp, sqrt(square(AXp) + square(AYp)))/(PI/180.0);         // ve vodorovne pozici se ze tri os spocte uhel, ktery bude slouzit k premapovani na 0 stupnu

    if (dbg == true)
      {
        Serial.print("Horiz. : AXp=");
        Serial.print(AXp);
        Serial.print("     AYp=");
        Serial.print(AYp);
        Serial.print("     AZp=");
        Serial.print(AZp);
        Serial.print("     Kal0=");
        Serial.println(nak_kalibrace0);
      }



    Serial.println(lng233);                                              // "Postav SQM svisle"

    Serial.println(">>");
    while (Serial.available() == 0)  delay(100);                         //  cekani na odklepnuti
    while(Serial.available()) Serial.read() ;

    prumer50();
    nak_kalibrace90 = atan2(AZp, sqrt(square(AXp) + square(AYp)))/(PI/180.0);        // tento uhel bude slouzit k premapovani na 90 stupnu

    if (dbg == true)
      {
        Serial.print("Vert.  : AXp=");
        Serial.print(AXp);
        Serial.print("     AYp=");
        Serial.print(AYp);
        Serial.print("     AZp=");
        Serial.print(AZp);
        Serial.print("     Kal90=");
        Serial.println(nak_kalibrace90);
      }


  
    EEPROM_write_int(471 , offset_x);                                    // offset X
    EEPROM_write_int(473 , offset_y);                                    // offset Y
    EEPROM_write_int(475 , offset_z);                                    // offset Z
    EEPROM_write_int(477, 10000 + ((nak_kalibrace0  + 0.005) * 100));    // zaokrouhleny horizont v setinach stupnu zvetseny o 100 stupnu kvuli ukladani kladne hodnoty do EEPROM.
    EEPROM_write_int(479, 10000 + ((nak_kalibrace90 + 0.005) * 100));    // zaokrouhleny zenit v setinach stupnu zvetseny o 100 stupnu kvuli ukladani kladne hodnoty do EEPROM.

#if ulozeni_naklonomeru == 1 || ulozeni_naklonomeru == 2                 // modul s 8 vyvody, nataceni kolem osy X cip smeruje nahoru nebo dolu
    zen_0_lp = AXp;
    zen_0_pz = AZp;
    EEPROM_write_int(481, AXp);                                          // zrychleni v prvni ose (leva/prava) pri skutecnem zenitu (pouziva se pro vertikalni vodovahu)
    EEPROM_write_int(483, AZp);                                          // zrychleni v druhe ose (predni/zadni) pri skutecnem zenitu (pouziva se pro vertikalni vodovahu)
#endif

#if ulozeni_naklonomeru == 3 || ulozeni_naklonomeru == 4                 // 10-vyvodovy modul, cip nahoru
    zen_0_lp = AYp;
    zen_0_pz = AZp;
    EEPROM_write_int(481, AYp);                                          // zrychleni v prvni ose (leva/prava) pri skutecnem zenitu (pouziva se pro vertikalni vodovahu)
    EEPROM_write_int(483, AZp);                                          // zrychleni v druhe ose (predni/zadni) pri skutecnem zenitu (pouziva se pro vertikalni vodovahu)
#endif


  
    naklonomer_info(false);

    zobraz_text(28);                                                     // "     " zhasnuti displeje

    Serial.println("\nOK\n");
    
  }
//----------------------------------------------



//----------------------------------------------
// rucni zadavani offsetu - dokud nejsou vystupy AXp a AYp co nejbliz 0 a vystup AZp co nejbliz cislu 16384 (nebo -16384 pokud cip smeruje v horizontalni pozici dolu)
//  prikazy:   "x 1234"  ..... prime zadani offsetu pro osu X (cislo muze byt zaporne)
//             "y 1234"  ..... prime zadani offsetu pro osu Y (cislo muze byt zaporne)
//             "z 1234"  ..... prime zadani offsetu pro osu Z (cislo muze byt zaporne)
//             "x+"      ..... pridani jednicky k offsetu pro osu X
//             "y+"      ..... pridani jednicky k offsetu pro osu Y
//             "z+"      ..... pridani jednicky k offsetu pro osu Z
//             "x-"      ..... odecteni jednicky od offsetu pro osu X
//             "y-"      ..... odecteni jednicky od offsetu pro osu Y
//             "z-"      ..... odecteni jednicky od offsetu pro osu Z
//             "q"       ..... ukonceni zadavani a prechod do vertikalni faze
void kalibrace_naklon_rucne(void)
  {
    zobraz_text(47);                                                     // "n-CAL" (kalibrace naklonomeru)
    Serial.println(lng232);                                              // "Poloz SQM horizontalne"

    char znak = ' ';
    char pomznak;

    while (znak != 'q')                                                  // rucni kalibrace horizontalni pozice konci pri zadani znaku 'q'
      {
        if (Serial.available())
          {
            delay(20);
            znak = Serial.read();                                        // ocekava se jmeno osy (x, y, z)
            pomznak = Serial.peek();                                     // ocekava se +, -, nebo cokoliv jineho
            if (pomznak == '+')
              {
                if (znak == 'x') offset_x ++;
                if (znak == 'y') offset_y ++;
                if (znak == 'z') offset_z ++;
              }            
            else if (pomznak == '-')
              {
                if (znak == 'x') offset_x --;
                if (znak == 'y') offset_y --;
                if (znak == 'z') offset_z --;
              }            
            else
              {
                if (znak == 'x') offset_x = Serial.parseInt();
                if (znak == 'y') offset_y = Serial.parseInt();
                if (znak == 'z') offset_z = Serial.parseInt();
              }

            while (Serial.available())  Serial.read();
            
            zapis_offset(offset_x , 6);
            zapis_offset(offset_y , 8);
            zapis_offset(offset_z , 10);
          }

        prumer50();
        
        Serial.print("offX=");
        Serial.print(offset_x);
        Serial.print("    offY=");
        Serial.print(offset_y);
        Serial.print("    offZ=");
        Serial.print(offset_z);

        Serial.print("        \tAXp=");
        Serial.print(AXp);
        Serial.print("    \tAYp=");
        Serial.print(AYp);
        Serial.print("    \tAZp=");
        Serial.println(AZp);
        delay(200);          
      }

    Serial.println(lng233);                                              // "Postav SQM svisle"

    Serial.println(">>");
    while (Serial.available() == 0)  delay(100);                         //  cekani na odklepnuti
    while(Serial.available()) Serial.read() ;

    prumer50();
    nak_kalibrace90 = atan2(AZp, sqrt(square(AXp) + square(AYp)))/(PI/180.0);        // tento uhel bude slouzit k premapovani na 90 stupnu

    EEPROM_write_int(471 , offset_x);                                    // offset X
    EEPROM_write_int(473 , offset_y);                                    // offset Y
    EEPROM_write_int(475 , offset_z);                                    // offset Z
    EEPROM_write_int(477, 10000 + ((nak_kalibrace0  + 0.005) * 100));    // zaokrouhleny horizont v setinach stupnu zvetseny o 100 stupnu kvuli ukladani kladne hodnoty do EEPROM.
    EEPROM_write_int(479, 10000 + ((nak_kalibrace90 + 0.005) * 100));    // zaokrouhleny zenit v setinach stupnu zvetseny o 100 stupnu kvuli ukladani kladne hodnoty do EEPROM.

#if ulozeni_naklonomeru == 1 || ulozeni_naklonomeru == 2                 // modul s 8 vyvody, nataceni kolem osy X cip smeruje nahoru nebo dolu
    zen_0_lp = AXp;
    zen_0_pz = AZp;
    EEPROM_write_int(481, AXp);                                          // zrychleni v prvni ose (leva/prava) pri skutecnem zenitu (pouziva se pro vertikalni vodovahu)
    EEPROM_write_int(483, AZp);                                          // zrychleni v druhe ose (predni/zadni) pri skutecnem zenitu (pouziva se pro vertikalni vodovahu)
#endif

#if ulozeni_naklonomeru == 3 || ulozeni_naklonomeru == 4                 // 10-vyvodovy modul, cip nahoru
    zen_0_lp = AYp;
    zen_0_pz = AZp;
    EEPROM_write_int(481, AYp);                                          // zrychleni v prvni ose (leva/prava) pri skutecnem zenitu (pouziva se pro vertikalni vodovahu)
    EEPROM_write_int(483, AZp);                                          // zrychleni v druhe ose (predni/zadni) pri skutecnem zenitu (pouziva se pro vertikalni vodovahu)
#endif


    naklonomer_info(false);
    zobraz_text(28);                                                     // "     " zhasnuti displeje
    Serial.println("\nOK\n");
    
  }



//----------------------------------------------
// dvouosa vodovaha v horizontalni pozici
void vodovaha_hor(void)
  {
    byte maskaX[] = {0,0,0,0,0};
    byte maskaY[] = {0,0,0,0,0};
    
    uhel();

#if ulozeni_naklonomeru == 1                                             // modul s 8 vyvody, nataceni kolem osy X cip smeruje nahoru
    if (AXp >  50)        maskaX[3] = 48;                                // nakloneno moc vpravo      
    if (AXp < -50)        maskaX[1] =  6;                                // nakloneno moc vlevo

    if (AXp >= -50 and AXp <= 50)                                        // vyrovnano v ose vlevo / vpravo
      {
        maskaX[3] = 48;     
        maskaX[1] =  6;     
      }
    
    if (AYp >  50)                                                       // nakloneno moc dopredu
      {
        maskaY[3] = 8;
        maskaY[2] = 8;
        maskaY[1] = 8;
      }
    if (AYp <  -50)                                                      // nakloneno moc dozadu
      {
        maskaY[3] =  1;
        maskaY[2] =  1;
        maskaY[1] =  1;
      }

    if (AYp >= -50 and AYp <= 50)                                        // vyrovnano v ose dopredu / dozadu
      {
        maskaY[3] = 9;        
        maskaY[2] = 9;
        maskaY[1] = 9;
      }
#endif



#if ulozeni_naklonomeru == 2                                             // modul s 8 vyvody, nataceni kolem osy X cipem dolu
    if (AXp < -50)        maskaX[3] = 48;                                // nakloneno moc vpravo      
    if (AXp >  50)        maskaX[1] =  6;                                // nakloneno moc vlevo

    if (AXp >= -50 and AXp <= 50)                                        // vyrovnano v ose vlevo / vpravo
      {
        maskaX[3] = 48;     
        maskaX[1] =  6;     
      }
    
    if (AYp >  50)                                                       // nakloneno moc dopredu
      {
        maskaY[3] = 8;
        maskaY[2] = 8;
        maskaY[1] = 8;
      }
    if (AYp <  -50)                                                      // nakloneno moc dozadu
      {
        maskaY[3] =  1;
        maskaY[2] =  1;
        maskaY[1] =  1;
      }

    if (AYp >= -50 and AYp <= 50)                                        // vyrovnano v ose dopredu / dozadu
      {
        maskaY[3] = 9;        
        maskaY[2] = 9;
        maskaY[1] = 9;
      }
#endif




#if ulozeni_naklonomeru == 3                                             // 10-vyvodovy modul / cip nahoru / prilepeny na piskaku
    if (AYp >   50)       maskaX[3] = 48;                                // nakloneno moc vpravo      
    if (AYp <  -50)       maskaX[1] =  6;                                // nakloneno moc vlevo

    if (AYp >= -50 and AYp <= 50)                                        // vyrovnano v ose vlevo / vpravo
      {
        maskaX[3] = 48;     
        maskaX[1] =  6;     
      }
    
    if (AZp < -50)                                                       // nakloneno moc dopredu
      {
        maskaY[3] = 8;
        maskaY[2] = 8;
        maskaY[1] = 8;
      }
    if (AZp >  50)                                                       // nakloneno moc dozadu
      {
        maskaY[3] =  1;
        maskaY[2] =  1;
        maskaY[1] =  1;
      }

    if (AZp >= -50 and AZp <= 50)                                        // vyrovnano v ose dopredu / dozadu
      {
        maskaY[3] = 9;        
        maskaY[2] = 9;
        maskaY[1] = 9;
      }
#endif


#if ulozeni_naklonomeru == 4                                             // 10-vyvodovy modul / cip nahoru / pripajeny na kolikach
    if (AYp <  -50)       maskaX[3] = 48;                                // nakloneno moc vpravo      
    if (AYp >   50)       maskaX[1] =  6;                                // nakloneno moc vlevo

    if (AYp >= -50 and AYp <= 50)                                        // vyrovnano v ose vlevo / vpravo
      {
        maskaX[3] = 48;     
        maskaX[1] =  6;     
      }
    
    if (AZp >  50)                                                       // nakloneno moc dopredu
      {
        maskaY[3] = 8;
        maskaY[2] = 8;
        maskaY[1] = 8;
      }
    if (AZp < -50)                                                       // nakloneno moc dozadu
      {
        maskaY[3] =  1;
        maskaY[2] =  1;
        maskaY[1] =  1;
      }

    if (AZp >= -50 and AZp <= 50)                                        // vyrovnano v ose dopredu / dozadu
      {
        maskaY[3] = 9;        
        maskaY[2] = 9;
        maskaY[1] = 9;
      }
#endif



      
    for (i = 0; i < 5 ; i++)
      {
        D_pamet[i] = maskaX[i] | maskaY[i] ;
      }
    aktualizuj_displej();
    
    delay(50);
    
  }
//----------------------------------------------



//----------------------------------------------
// dvouosa vodovaha v zenitove pozici
void vodovaha_zen(void)
  {
    byte maskaX[] = {0,0,0,0,0};
    byte maskaZ[] = {0,0,0,0,0};
    
    uhel();

#if ulozeni_naklonomeru == 1                                             // modul s 8 vyvody, nataceni kolem osy X cip nahoru
    if (AXp >  zen_0_lp + 50)        maskaX[4] = 33;                     // nakloneno moc vpravo
    if (AXp <  zen_0_lp - 50)        maskaX[0] =  3;                     // nakloneno moc vlevo        

    if (AXp >= zen_0_lp - 50 and AXp <= zen_0_lp + 50)                   // osa vlevo / vpravo vyrovnana
      {
        maskaX[4] = 33;     
        maskaX[0] =  3;     
      }

    if (AZp > zen_0_pz + 50)                       maskaZ[2] = 28;       // nakloneno moc dozadu (znak 'u')       
    if (AZp < zen_0_pz - 50)                       maskaZ[2] = 35;       // nakloneno moc dopredu (znak '^')

    if (AZp >= zen_0_pz - 50 and AZp <= zen_0_pz + 50)  maskaZ[2] = 63;  // osa dopredu / dozadu vyrovnana
#endif


#if ulozeni_naklonomeru == 2                                             // modul s 8 vyvody, nataceni kolem osy X cip smeruje dolu
    if (AXp < zen_0_lp - 50)        maskaX[4] = 33;                      // nakloneno moc vpravo
    if (AXp > zen_0_lp + 50)        maskaX[0] =  3;                      // nakloneno moc vlevo        

    if (AXp >= zen_0_lp - 50 and AXp <= zen_0_lp + 50)                   // osa vlevo / vpravo vyrovnana
      {
        maskaX[4] = 33;     
        maskaX[0] =  3;     
      }

    if (AZp < zen_0_pz - 50)                       maskaZ[2] = 28;       // nakloneno moc dozadu (znak 'u')       
    if (AZp > zen_0_pz + 50)                       maskaZ[2] = 35;       // nakloneno moc dopredu (znak '^')

    if (AZp >= zen_0_pz - 50 and AZp <= zen_0_pz + 50)  maskaZ[2] = 63;  // osa dopredu / dozadu vyrovnana   
#endif



#if ulozeni_naklonomeru == 3 || ulozeni_naklonomeru == 4                 // modul s 10 vyvody, nataceni kolem osy Y cip nahoru
    if (AYp < zen_0_lp - 50)        maskaX[4] = 33;                      // nakloneno moc vpravo
    if (AYp > zen_0_lp + 50)        maskaX[0] =  3;                      // nakloneno moc vlevo        

    if (AYp >= zen_0_lp - 50 and AYp <= zen_0_lp + 50)                   // osa vlevo / vpravo vyrovnana
      {
        maskaX[4] = 33;     
        maskaX[0] =  3;     
      }

    if (AZp > zen_0_pz + 50)                       maskaZ[2] = 28;       // nakloneno moc dozadu (znak 'u')       
    if (AZp < zen_0_pz - 50)                       maskaZ[2] = 35;       // nakloneno moc dopredu (znak '^')

    if (AZp >= zen_0_pz - 50 and AZp <= zen_0_pz + 50)  maskaZ[2] = 63;  // osa dopredu / dozadu vyrovnana   
#endif




    
    for (i = 0; i < 5 ; i++)
      {
        D_pamet[i] = maskaX[i] | maskaZ[i] ;
      }



    aktualizuj_displej();
    
    delay(50);
    
  }
//----------------------------------------------





//----------------------------------------------
// Servisni informace o aktualnich offsetech a krajnich kalibracnich bodech
// Pri parametru 'true' vypise jeste 10x obsah vsech akcelaracnich registru a jejich prepocet na uhel. 
void naklonomer_info(boolean test)
  {
    Serial.print("Offset_X: ");
    Serial.println(offset_x);
    Serial.print("Offset_Y: ");
    Serial.println(offset_y);
    Serial.print("Offset_Z: ");
    Serial.println(offset_z);

    Serial.print("Zen_0_L/P: ");
    Serial.println(zen_0_lp);

    Serial.print("Zen_0_P/Z: ");
    Serial.println(zen_0_pz);


    Serial.print("Horizont: ");
    Serial.println(nak_kalibrace0);
    Serial.print("Zenit: ");
    Serial.println(nak_kalibrace90);

    if (test == true)
      {
        for (nak_cyklus = 0 ; nak_cyklus < 10 ; nak_cyklus++)
          {
            int16_t pom_uhel = uhel(); 
            Serial.print("AXp: ");
            Serial.print(AXp);
            Serial.print("         AYp: ");
            Serial.print(AYp);
            Serial.print("         AZp: ");
            Serial.print(AZp);        
            Serial.print("         ang Z: ");
      
            Serial.print(pom_uhel);        
            Serial.print("    (");
            Serial.print((pom_uhel-10000.0)/10.0);        
            Serial.println("')");
          }
      }
  }
//----------------------------------------------




//----------------------------------------------
// zapis dvojbajtoveho cisla do zadaneho registru (a registru +1) MPU6050 
void zapis_offset( int16_t hodnota, int16_t registr)
  {
    Wire.beginTransmission (I2C_ADDR_NAKLON);
    Wire.write (registr);
    Wire.write(hodnota / 256);
    Wire.endTransmission (true);
    delay(20);
    Wire.beginTransmission (I2C_ADDR_NAKLON); 
    Wire.write (registr +1);
    Wire.write(hodnota % 256);
    Wire.endTransmission (true);
    delay(20);                    
  }
//----------------------------------------------



//----------------------------------------------
// druha mocnina pro velka cisla
int32_t square(int16_t vstup)
  {
    return  1UL * vstup * vstup;
  }
//----------------------------------------------



//----------------------------------------------
// nahrada funkce map() pro desetinna cisla
float mapfloat(float x, float in_min, float in_max, float out_min, float out_max)
  {
    return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
  }
//----------------------------------------------

#endif  
